<?php
/***************************************
** Filename.......: email.reply.redirect.php
** Project........: Frederick
** Last Modified..: 10 December 2001
***************************************/

	include('config.php');

/***************************************
** Login check
***************************************/

	auth::check_login('login.php', 'email.list.php');

/***************************************
** Some functions specific to this script
***************************************/

	/***************************************
    ** Returns the body for a reply-to,
	** reply-to-all and forward
    ***************************************/

	function _get_body($strip_sig = TRUE){
		global $CONFIG, $USERPREFS, $msg_struct, $structure, $SESSION;

		if(!empty($msg_struct->body_plain)){
			$body = $msg_struct->body_plain;
		}elseif(!empty($msg_struct->body_html)){
			$body = strip_tags($msg_struct->body_html);
		}
		// Snip anything after a sigsep
		if($strip_sig AND ($pos = strpos($body, "\r\n-- \r\n")) !== FALSE)
			$body = substr($body, 0, $pos);

		// Prefix text with >
		$body = preg_replace('/^(.)/m', $USERPREFS['settings']['reply_indent_char'].' \1', wordwrap($body, 80));

		// Parse From: header
		if(!empty($structure->headers['from'])){
			include_once($CONFIG['includes'].'RFC822.php');
			$from_addresses = Mail_RFC822::parseAddressList($structure->headers['from'], $SESSION['email']['smtp_helo'], TRUE);
	
			// Add xxx quoted...
			if(count($from_addresses) > 1){
				$personal = !empty($from_addresses[0]->personal) ? '"'.$from_addresses[0]->personal.'" ' : '';
				$quote = $personal.'<'.$from_addresses[0]->mailbox.'@'.$from_addresses[0]->host.'>';
			}else{
				$quote = $structure->headers['from'];
			}
		}else{
			$quote = '';
		}
		
		$body = $quote.' wrote:'."\r\n\r\n".$body;

		return $body;
	}

	/***************************************
    ** Returns list-post address (from List-Post:
	** header).
    ***************************************/
	
	function _get_list_post(){
		global $structure;
		if(!empty($structure->headers['list-post']) AND preg_match('/^<mailto:(.*)>/Ui', $structure->headers['list-post'], $matches)){
			$ret = $matches[1];
		}else{
			$ret = _get_to();
		}
		
		return $ret;
	}

	/***************************************
    ** Returns Subject, Body and unsubscribe
	** address based on List-Unsubscribe header
    ***************************************/
	
	function  _get_list_unsubscribe(){
		global $structure, $CONFIG;

		if(!empty($structure->headers['list-unsubscribe']) AND preg_match('/^<(mailto:.*)>/Ui', $structure->headers['list-unsubscribe'], $matches)){
			include_once($CONFIG['includes'] . 'URL.php');
			$url =& new Net_URL($matches[1]);

			$email   = $url->path;
			$subject = !empty($url->querystring['subject']) ? $url->querystring['subject'] : 'UNSUBSCRIBE';
			$body    = !empty($url->querystring['body'])    ? $url->querystring['body']    : 'UNSUBSCRIBE';

		}else{
			$email   = _get_to();
			$subject = 'UNSUBSCRIBE';
			$body    = 'UNSUBSCRIBE';
		}
		
		return array($email, $subject, $body);
	}

	/***************************************
    ** Returns a To: header for reply-to and
	** reply-to-all
    ***************************************/

	function _get_to($reply_all = FALSE){
		global $structure, $CONFIG, $SESSION, $USERPREFS;
		include_once($CONFIG['includes'].'RFC822.php');

		$email_address = isset($USERPREFS['settings']['email_address']) ? $USERPREFS['settings']['email_address'] : $SESSION['email']['user'].'@'.$SESSION['email']['srvr'];
		$email_address = Mail_RFC822::parseAddressList($email_address, $SESSION['email']['srvr'], TRUE);
		$email_address = $email_address[0];

		$from_header = isset($structure->headers['from']) ? $structure->headers['from'] : '';
		$from_addresses = Mail_RFC822::parseAddressList($from_header, $SESSION['email']['smtp_helo'], TRUE);

		if(isset($structure->headers['reply-to'])){
			$to[] = $structure->headers['reply-to'];
	
		}elseif(count($from_addresses) > 1 AND isset($structure->headers['sender']) AND !$reply_all){
			$to[] = $structure->headers['sender'];
	
		}elseif(count($from_addresses) > 1){
			if(!$reply_all){
				if(preg_match('/^".+"$/', $to_addresses[$i]->personal))
					$to_addresses[$i]->personal = substr($to_addresses[$i]->personal, 1, -1);
				$personal = !empty($from_addresses[0]->personal) ? '"'.$from_addresses[0]->personal.'" ' : '';
				$to[]     = $personal.'<'.$from_addresses[0]->mailbox.'@'.$from_addresses[0]->host.'>';
			}else{
				for($i=0, $cnt = count($from_addresses); $i<$cnt; $i++){
					if(preg_match('/^".+"$/', $to_addresses[$i]->personal))
						$to_addresses[$i]->personal = substr($to_addresses[$i]->personal, 1, -1);
					$personal = !empty($from_addresses[$i]->personal) ? '"'.$from_addresses[$i]->personal.'" ' : '';
					$to[]     = $personal.'<'.$from_addresses[$i]->mailbox.'@'.$from_addresses[$i]->host.'>';		
				}
			}

		}else{
			if(!$reply_all){
				$to[] = $from_header;
			}else{
				$to[] = $from_header;
			}
		}

		if($reply_all){
			if(isset($structure->headers['to'])){
				$to_addresses = Mail_RFC822::parseAddressList($structure->headers['to'], $SESSION['email']['smtp_helo'], TRUE);

				for($i=0, $cnt = count($to_addresses); $i<$cnt; $i++){
					if($to_addresses[$i]->mailbox.'@'.$to_addresses[$i]->host == $email_address)
						continue;
					if(preg_match('/^".+"$/', $to_addresses[$i]->personal))
						$to_addresses[$i]->personal = substr($to_addresses[$i]->personal, 1, -1);
					$personal = !empty($to_addresses[$i]->personal) ? '"'.$to_addresses[$i]->personal.'" ' : '';
					$to[]     = $personal.'<'.$to_addresses[$i]->mailbox.'@'.$to_addresses[$i]->host.'>';		
				}
			}
		}

		return implode(', ', $to);
	}

	/***************************************
    ** Returns the Cc: header for reply-to-all
    ***************************************/

	function _get_cc(){

		global $CONFIG, $SESSION, $structure;

		if(!empty($structure->headers['cc'])){
			include_once($CONFIG['includes'].'RFC822.php');
			$cc_addresses = Mail_RFC822::parseAddressList($structure->headers['cc'], $SESSION['email']['smtp_helo'], TRUE);
			for($i=0, $cnt = count($cc_addresses); $i<$cnt; $i++){
				if(preg_match('/^".+"$/', $cc_addresses[$i]->personal))
					$cc_addresses[$i]->personal = substr($cc_addresses[$i]->personal, 1, -1);
				$personal = !empty($cc_addresses[$i]->personal) ? '"'.$cc_addresses[$i]->personal.'" ' : '';
				$cc[]     = $personal.'<'.$cc_addresses[$i]->mailbox.'@'.$cc_addresses[$i]->host.'>';		
			}

			return implode(', ', $cc);
		}
	}

	/***************************************
    ** Returns the Subejct: header for a
	** reply-to, reply-to-all, or forward
    ***************************************/

	function _get_subject($pretext = 'Re: '){
		global $structure;

		$subject = isset($structure->headers['subject']) ? $structure->headers['subject'] : '';

		// Stip Re: or Fw: or Aw:
		while(preg_match('/^\s*(re:|aw:|fw:)\s*/i', $subject)){
			$subject = preg_replace('/^\s*(re:|aw:|fw:)\s*/i', '', $subject);
		}

		$subject = $pretext.$subject;
		return $subject;
	}

	/***************************************
    ** Returns some headers gotten from the
	** mail we're replying to. Namely,
	** In-Reply-To: and References:
    ***************************************/

	function _get_headers(){
		global $structure;

		$headers = array();
		if(isset($structure->headers['message-id'])){
			$message_id = $structure->headers['message-id'];
			$headers['In-Reply-To'] = $message_id;
			$headers['References']  = isset($structure->headers['references']) ? rtrim($structure->headers['references']).' '.$structure->headers['message-id'] : $message_id;
		}

		return $headers;
	}

/***************************************
** Check for required inputs
***************************************/

	$error = FALSE;
	if(!input::validate(@$HGV, 'action')){
		$error = TRUE;
		$error_msg = lang('invalid_action');
	}

	if(!input::validate(@$HGV, 'msg_id')){
		$error = TRUE;
		$error_msg = lang('invalid_msg_id');
	}

/***************************************
** Perform an action if no error
***************************************/

	if(!$error){

		$msg_id = $HGV['msg_id'];
		$partno = isset($HGV['partno']) ? $HGV['partno'] : NULL;

		/***************************************
		** Get details of the message.
		***************************************/

		if($mailaccess->connect($SESSION['email']['srvr'], $SESSION['email']['user'], base64_decode($SESSION['email']['pass']), $SESSION['email']['port'], $SESSION['email']['type'], $SESSION['email']['mbox'])){

			$email  = $mailaccess->get_raw_headers($msg_id, $partno) . $mailaccess->get_body($msg_id, $partno, TRUE);
			$email  = preg_replace("/\r?\n/", "\r\n", $email);

			if($HGV['action'] !== 'forward-attach'){
				$params = array(
								'input'          => $email,
								'crlf'           => "\r\n",
								'include_bodies' => TRUE,
								'decode_headers' => TRUE,
								'decode_bodies'  => TRUE
								);
	
				include($CONFIG['includes'].'mimeDecode.php');	
				$structure  = Mail_mimeDecode::decode($params);

				// Ensure To and Cc are not arrays:
				if(@is_array($structure->headers['to'])){
					$structure->headers['to'] = implode(', ', $structure->headers['to']);
				}

				if(@is_array($structure->headers['cc'])){
					$structure->headers['cc'] = implode(', ', $structure->headers['cc']);
				}

				$msg_struct = new message();
				get_msg($structure, $msg_id, (isset($partno) ? $partno : 1) , FALSE, FALSE);
			}

			
			/***************************************
            ** Which action to perform?
            ***************************************/

			$redirect_to = 'email.compose.php';
			$headers = $mailaccess->headerinfo($msg_id);

			switch($HGV['action']){
		
				case 'reply':
					$SESSION['email_compose']['headers'] = _get_headers();
					$SESSION['email_compose']['subject'] = _get_subject();
					$SESSION['email_compose']['to']      = _get_to();
					$SESSION['email_compose']['text']    = _get_body();
					break;

				case 'reply-all':
					$SESSION['email_compose']['headers'] = _get_headers();
					$SESSION['email_compose']['subject'] = _get_subject();
					$SESSION['email_compose']['to']      = _get_to(TRUE);
					$SESSION['email_compose']['cc']      = _get_cc();
					$SESSION['email_compose']['text']    = _get_body();
					break;

				case 'reply-list':
					$SESSION['email_compose']['headers'] = _get_headers();
					$SESSION['email_compose']['subject'] = _get_subject();
					$SESSION['email_compose']['to']      = _get_list_post();
					$SESSION['email_compose']['text']    = _get_body();
					break;

				case 'unsubscribe':
					list($to, $subject, $body) = _get_list_unsubscribe();
					$SESSION['email_compose']['to']      = $to;
					$SESSION['email_compose']['subject'] = $subject;
					$SESSION['email_compose']['text']    = $body;
					break;

				case 'forward':
					$SESSION['email_compose']['subject'] = _get_subject('Fw: ');
					$SESSION['email_compose']['text']    = _get_body(FALSE);
					break;

				case 'forward-attach':
					// Write email to tmp file
					$filename = fred::get_tmp_filename();
					if(fred::write_file($filename, $email)){

						$SESSION['email_compose']['subject'] = _get_subject('Fw: ');
						$SESSION['email_compose']['files'][] = array(
																		'tmp_name' => $filename,
																		'size'     => strlen($email),
																		'c_type'   => 'message/rfc822',
																		'filename' => 'Attached message',
																		'encoding' => '7bit'
																	);
					}
					// FIXME Error msg?
					break;

				default:
					$redirect_to = 'email.list.php';
					$SESSION['common']['error_msg'] = lang('invalid_action');
					break;
			}

		/***************************************
        ** Connection mail server failed
        ***************************************/

		}else{
			$SESSION['common']['error_msg'] = lang('mail_connection_error');
			$redirect_to = 'email.list.php';
		}

	}else{

		$redirect_to = 'email.list.php';
		$SESSION['common']['error_msg'] =& $error_msg;
	}

/***************************************
** Redirect
***************************************/

	fred::redirect($redirect_to);
?>