<?php
/***************************************
** Filename.......: class.fred.inc
** Project........: Frederick
** Last Modified..: 04 December 2001
** Copyright......: Richard Heyes 2001
***************************************/

/***************************************
** Base class containing common functions
***************************************/

class fred {

	/***************************************
    ** Checks for an error message in the 
	** session, and unsets it if it's there.
    ***************************************/

	function error_msg(&$error_msg, $no_error = ''){
		global $SESSION;

		if(isset($SESSION['common']['error_msg'])){
			$error_msg = $SESSION['common']['error_msg'];
			unset($SESSION['common']['error_msg']);
		}else{
			$error_msg = $no_error;
		}
	}
	
	/***************************************
    ** Redirect function issues a Location:
	** header and exits. Optionally sets an 
	** error message in the session.
    ***************************************/

	function redirect($to, $error_msg = NULL){
		global $SESSION, $CONFIG;

		if(isset($error_msg)){
			$SESSION['common']['error_msg'] = $error_msg;
		}
		
		header('Location: '.$CONFIG['web_path'].$to);
		exit;
	}
	
	/***************************************
    ** Returns a filename that doesn't exist
	** in the (optionally) supplied directory.
	** For tmp files. Directory is the application
	** tmp dir if not supplied.
    ***************************************/

	function get_tmp_filename($dir = NULL){
		global $CONFIG;

		if(!isset($dir)){
			$dir = $CONFIG['tmp_dir'];
		}

		return tempnam($dir, 'tmp.');
	}

	/***************************************
    ** Writes data (second arg) to given filename
	** (first arg), and applies perms (third arg).
    ***************************************/

	function write_file($filename, $data, $perms = 0755){

		$fp = fopen($filename, 'wb');
		if($fp){
			fwrite($fp, $data);
			fclose($fp);
			chmod($filename, $perms);
			return TRUE;

		}else{
			return FALSE;
		}
	}

	/***************************************
    ** Removes an entry from the given array
	** if it matches the given pcre regex.
    ***************************************/

	function preg_array_remove($regex, $array){

		for($i=0; $i<count($array); $i++){
			if(preg_match($regex, $array[$i])){
				unset($array[$i]);
			}
		}
		
		return array_values($array);
	}
}

/***************************************
** A class to encompass some useful array
** functions
***************************************/

class _array {

	/***************************************
    ** Inserts $data into $array at $index
    ***************************************/

	function insert($array, $index, $data){
		$end_of_array = array_splice($array, $index);
		$array = array_merge($array, $data, $end_of_array);
		return $array;
	}

	/***************************************
    ** Swaps the two indices $a and $b of $array
    ***************************************/

	function swap($array, $a, $b){
		if(isset($array[$a]) AND isset($array[$b])){
			$data_a    = $array[$a];
			$array[$a] = $array[$b];
			$array[$b] = $data_a;
		}

		return $array;
	}
}
?>