<?php
/***************************************
** Filename.......: class.prefs_file.inc
** Project........: Frederick
** Last Modified..: 03 November 2001
** Copyright......: Richard Heyes 2001
***************************************/

/***************************************
** This class uses files to store and
** retrieve the prefs. Serialises the data
** first and then writes to a file. The files
** are held in a 3 layer directory structure.
** The 3 layers are made up from the first
** 3 letters of the unique identifier. So
** prefs with unique id of "publicj" would
** be held in {data_dir}/p/u/b/publicj, whereas
** prefs with unique id of heyesr would be
** held in {data_dir}/h/e/y/heyesr
***************************************/

class prefs_file extends prefs{

	var $data_dir;

	/***************************************
    ** Constructor function. Takes the parameters
	** array passed to it from the factory function
    ***************************************/

	function prefs_file($params){

		$this->data_dir = $params['data_dir'];
	}

	/***************************************
    ** This function reads the prefs in from 
	** a file. If the second argument is FALSE
	** then the 3 level dir structure will not
	** be used, and the file will be read in
	** from {data_dir}
    ***************************************/

	function read($unique_id, $use_dirs = TRUE, $default_file = 'default.user'){
		global $USERPREFS, $HOSTINFO, $CONFIG;

		$path = $this->get_path($unique_id, $use_dirs);

		// If doesn't exist, create it
		if(!file_exists($path)){
			$default_file = file_exists($this->get_path('local.'.$default_file, FALSE)) ? 'local.'.$default_file : $default_file;
			$default = $this->get_path($default_file, FALSE);
			copy($default, $path);
		}

		if($path){
			$contents = fread(fopen($path, 'r'), filesize($path));

			// Try to unserialize, if that fails, try decrypting
			// and then unserializing.
			if($unserialized = @unserialize($contents)){
				return $unserialized;

			}elseif($unserialized = @unserialize($this->decrypt_data($contents))){
				return $unserialized;

			}else{
				// FIXME Need logging here in case can't unserialize the prefs
				return FALSE;
			}

		}else{
			return FALSE;
		}
	}

	/***************************************
    ** This function writes the preferences
	** to a file.
    ***************************************/

	function write($unique_id, $data, $use_dirs = TRUE){
		global $HOSTINFO, $USERPREFS, $CONFIG;

		$path = $this->get_path($unique_id, $use_dirs);

		$serialized = serialize($data);

		// If unique_id is hostinfo, make sure usage of dirs is true to ensure
		// we're not saving host information.
		if($unique_id != 'hostinfo' OR $use_dirs == TRUE){
			if($HOSTINFO['settings']['encrypt_user_data'] == '1'){
				include_once($CONFIG['includes'].'class.crypto.inc');
				$crypto     =& crypto::factory($CONFIG['crypto_key']);
				$serialized =  $crypto->encrypt($serialized);

			}elseif($HOSTINFO['settings']['encrypt_user_data'] == 'user' AND $USERPREFS['settings']['encrypt_user_data']){
				include_once($CONFIG['includes'].'class.crypto.inc');
				$crypto     =& crypto::factory($CONFIG['crypto_key']);
				$serialized =  $crypto->encrypt($serialized);
			}
		}elseif($unique_id == 'hostinfo' AND file_exists($this->data_dir . 'local.hostinfo')){
			$path = $this->get_path('local.hostinfo', FALSE);
		}

		return fwrite(@fopen($path, 'w'), $serialized);
	}

	/***************************************
    ** This function, given a unique_id, will
	** return the full path to the prefs file.
    ***************************************/

	function get_path($unique_id, $use_dirs){
		global $CONFIG;

		$data_dir = substr($this->data_dir, -1) == DIRECTORY_SEPARATOR ? $this->data_dir : $this->data_dir . DIRECTORY_SEPARATOR;

		$dir1 = substr($unique_id,0,1);
		$dir2 = substr($unique_id,1,1);
		$dir3 = substr($unique_id,2,1);
		$path = $data_dir;

		// Use 3 level directory struture?
		if(!$use_dirs){
			return $path.$unique_id;
		}

		$path .= $dir1;

		if(!file_exists($path)){
			mkdir($path, 0775);
			chmod($path, 0775);
		}

		$path .= DIRECTORY_SEPARATOR . $dir2;

		if(!file_exists($path)){
			mkdir($path, 0775);
			chmod($path, 0775);
		}

		$path .= DIRECTORY_SEPARATOR . $dir3;

		if(!file_exists($path)){
			mkdir($path, 0775);
			chmod($path, 0775);
		}

		return (@is_writeable($path)) ? $path . DIRECTORY_SEPARATOR . $unique_id : FALSE;
	}

	/***************************************
    ** Decrypt data. This function used by
	** $this->read()
    ***************************************/

	function decrypt_data($data){
		global $CONFIG;

		include_once($CONFIG['includes'].'class.crypto.inc');
		$crypto =& crypto::factory($CONFIG['crypto_key']);
		return $crypto->decrypt($data);
	}
}
?>