<?php
/***************************************
** Filename.......: class.tree_node.inc
** Project........: Frederick Webmail
** Last Modified..: 02 December 2001
** Copyright......: Richard Heyes 2001
***************************************/

class tree_node {

	var $data;
	var $children;

	/***************************************
    ** Constructor
    ***************************************/

	function tree_node($data){
		$this->data     = $data;
		$this->children = array();
	}

	/***************************************
    ** add_child()
    ***************************************/
	
	function &add_child($node){
		$this->children[] = $node;
		return $this->children[count($this->children)-1];
	}
	
	/***************************************
    ** remove_child()
    ***************************************/
	
	function remove_child($id){
		if(isset($this->children[$id]))
			unset($this->children[$id]);
	}

	/***************************************
    ** num_children()
    ***************************************/

	function num_children(){
		$count = count($this->children);
		for($i=0; $i<count($this->children); $i++){
			$count += $this->children[$i]->num_children();
		}
		return $count;
	}

	/***************************************
    ** num_children_immediate()
    ***************************************/

	function num_children_immediate(){
		return count($this->children);
	}

	/***************************************
    ** walk()
    ***************************************/
	
	function walk($function){
		$function($data);
		for($i=0; $i<count($children); $i++){
			$children[$i]->walk($function);
		}
	}
	
	/***************************************
    ** find()
    ***************************************/
	
	function &find($member, $data){

		if($this->data[$member] === $data){
			return $this;

		}elseif(count($this->children)){
			for($i=0; $i<count($this->children);$i++){
				if(is_object($this->children[$i])){
					if(($node =& $this->children[$i]->find($member, $data)) !== FALSE){
						return $node;
					}
				}
			}
			return FALSE;

		}else{
			return FALSE;
		}
	}

	/***************************************
    ** return_all()
    ***************************************/
	
	function return_all($mailaccess, $indent){
		$return = array();
		$mailaccess->get_msg_summary($this->data['msg_id'], $return);
		
		$return[0] = array_merge($return[0], array('indent' => str_repeat('&nbsp;&nbsp;&nbsp;', $indent)));
		foreach($this->children as $value){
			$return = array_merge($return, $value->return_all($mailaccess, $indent + 1));
		}

		return $return;
	}
	
	/***************************************
    ** clean()
    ***************************************/
	
	function clean(){
		$tmp = array();
		foreach($this->children as $value){
			if(is_object($value)){
				$value->clean();
				$tmp[] = $value;
			}
		}
		$this->children = $tmp;
	}
}
?>